#!/usr/bin/perl -w
#
# Get TV listings for http://www.bleb.org/tv/, and generates Perl code
# containing the data.
#
# (c) Andrew Flegg 2000. Thanks to Graham Chapman for further work.

use strict;
use lib "/home/jaffa/bin/tv";
use Telly;
use LWP::Simple;
use vars qw($DEBUG $WEEK);

my $cli = ":" . join(":", @ARGV) . ":";
$DEBUG = ($cli =~ m/:-debug:/i);
$WEEK  = ($cli =~ m/:-week:/i);

my $LOOKING_FOR_FIRST = 0;
my $LOOKING_FOR_TIME  = 1;
my $LOOKING_FOR_TITLE = 2;
my $READING_DESC      = 3;

# -- Process each channel in a separate thread...
#
foreach my $i (keys(%Telly::CHANNELS)) {
    my $j     = $Telly::CHANNELS{$i};
    my $child = 0; # fork();
#    $child = fork();

    if ($child == 0) {
        my $dir = $DATA;
        &doChannel("$BASE$i", "$dir/+0", $j);
        if ($WEEK) {
	    for(my $day=1;$day<=6;$day++) {
                my $lcl = "$BASE$i";
                eval('$lcl =~ s/day1/day'.($day+1).'/;');
		print "$lcl => +$day/$j\n" if $DEBUG;
                &doChannel($lcl, "$dir/+$day", $j);
            }
        }
    }
    sleep 2;
#    last if $j eq 'itv';
}
exit;


sub doChannel {
    my ($i, $dir, $j) = @_;

    my $data = get($i);
    print "Unable to fetch $i\n", return unless $data;
    open(OUT, ">$dir/$j.pm");
    if (! \*OUT) {
        print "getchannels: couldn't write $j.dat\n";
	return;
    }

    my @in   = split(/\n/, $data);
    print "Processing $i => $dir/$j...\n" if $DEBUG;
    my $date = scalar(localtime);
    print OUT <<EOH;
# Today's Telly information for $j
#
# Automatically generated at $date
# (c) Andrew Flegg 2000. All rights reserved.

my \@channel;
EOH

	# -- Process the input
        #
	my $status = $LOOKING_FOR_TIME;
        my $time   = "";
	my $title  = "";
	my $desc   = "";
        my $url    = "";
	my $count  = 0;
#	while (<IN>) {
	foreach (@in) {
	    if ($status == $LOOKING_FOR_TIME) {
		if (m!<td[^>]+width="109"[^>]*?>(<s[^>]+>\s*)*(\d{1,2}:\d{2})?\s*</span>!i) {
		    # -- Found a new program
		    #
		    if ($2) {
			$time = time24($2);
		    } else {
			$time = "";
		    }
		    $title  = "";
		    $desc   = "";
		    $status = $LOOKING_FOR_TITLE;
                    print "$time\t: " if $DEBUG;
		}
	    
	    } if ($status == $LOOKING_FOR_TITLE) {
                if (m!<a[^>]+href="tv_det.html\?t=(\d+)"[^>]*>!) {
                    # -- Found the "more details" URL
                    #
                    print "$1 " if $DEBUG;
                    $url = "info.cgi?id=$1";
                }

		if (m!<span class="title">(<s[^>]+>)*\s*(.*?)\s*(</[^>]+>)*?</span>!i) {
		    # -- Got the title
		    #
		    $title  = $2;
		    $status = $READING_DESC;
                    print "$title... " if $DEBUG;
		}

	    } if ($status == $READING_DESC) {
		if (m!<span class="desc">(<s[^>]+>)*\s*(.*?)\s*(</[^>]+>)*?</span>!i) {
		    # -- End of record
		    #
		    $desc = $2; #quotemeta($desc);
		    $title =~ s/'/\\'/g;
		    $desc  =~ s/'/\\'/g;

	            $desc .= " <small><b><a href=\"$url\">[&gt;&gt;]</a></b></small>" if $url;
		    print OUT "\$channel[$count]\t= ['$time',\n".
			      "         \t  '$title',\n".
			      "         \t  '$desc'];\n";
		    $status = $LOOKING_FOR_TIME;
		    $count++;
                    print "done.\n" if $DEBUG;
		}
	    }
	}

    print OUT "\n\n\$Telly::LISTINGS{'$j'} = \\\@channel;\n1;\n";
    close(OUT);
    print "+++ Warning: $dir/$j has no programmes\n" unless $count;
}



