#!/usr/bin/perl -w

# bleb.org/tv2 wap interface
# version 2.10.3.1
# versions numbered by rewritenumber.featurechange.cosmeticchange.bugfix

# Current $go are:
#  viewsdeck, views and channels
#  list, list of programmes, needs $ch
#  desc, description of programme, needs $ch + $pid
#  film, list of films, needs $view
# Current options are:
#  $day, $time

BEGIN { do '/home/jaffa/lib/loadperl.pm' }

use strict;
use TV::Core;
use TV::Format;
use TV::Search;
use POSIX qw(strftime);
use CGI qw(:standard);

# -- Name of the cgi program
my $self = "/waptv";

# -- Establish view arrays
my @views = ("terrestrial","freeview","digitalradio","radio","premium");
my %views = (
   "terrestrial"  => "Terrestrial",
   "freeview"     => "FreeView",
   "digitalradio" => "Digital Radio",
   "radio"        => "Radio",
   "premium"      => "Premium",
);
my %mapping = (
   terrestrial  => [ "bbc1","bbc2","itv1","ch4","five" ],
   freeview     => [ "bbc3","bbc4","bbc7","bbc_news24","british_eurosport",
                     "cbbc","cbeebies","itv2","ftn","uk_bright_ideas",
                     "uk_gold","uk_history","uk_horizons","uk_style" ],
   digitalradio => [ "bbc_6music","bbc_radio1_xtra" ],
   radio        => [ "bbc_radio1","bbc_radio2","bbc_radio3","bbc_radio4",
                     "bbc_radio5_live","bbc_radio5_live_sports_extra" ],
   premium      => [ "sky_one","sky_premier","boomerang","cartoon_network",
                     "discovery","e4","film_four","nickelodeon",
                     "paramount","scifi","sky_moviemax","sky_sports1",
                     "sky_sports3" ],
);

# -- Timeslot arrays
my @slots = (0, 1, 2, 3, 4, 5);
my %slots = (
   0 => ["All", ""],
   1 => ["Future", ""],
   2 => ["Morning", "06:00 - 12:00"],
   3 => ["Afternoon", "12:00 - 18:00"],
   4 => ["Evening", "18:00 - 00:00"],
   5 => ["Late Night", "00:00 - 06:00"],
);
my %tcols = (
   0 => ["x", "x"],
   1 => [(strftime "%R", localtime), "x"],
   2 => ["x", "12:00"],
   3 => ["12:00", "18:00"],
   4 => ["18:00", "24:00"], # we use a 48 hour clock
   5 => ["24:00", "x"],
);

# -- Figure out where to go, and work out defaults
my $go   = param('go')   || "viewsdeck";
my $view = param('view') || "terrestrial";
my $ch   = param('ch')   || "bbc1";
my $day  = param('day')  || 0;
my $time = param('time'); $time = 1 unless defined($time);
my $pid  = param('pid')  || 0;

# -- Don't expire this page from the wap cache
my $expire = 0;
my $obj  = TV::Format->new(channel => $ch, day => $day);

# -- Decide which page to call
if ($go eq "list") {
   $expire = 1;
   listing();
} elsif ($go eq "desc") {
   $expire = 1;
   description();
} elsif ($go eq "film") {
   $expire = 1;
   film();
} else { # $go eq "viewsdeck" or corrupt
   viewsdeck();
}

# -- Show the views deck ------------------------------------------
sub viewsdeck {
   wml_header();
   print "<wml>\n";

# -- Views card
   print "<card id=\"view\" title=\"";
   if ($time != 1) {
      print TV::Format->getDayName($day)." ".$slots{$time}[0];
   } else {
      print TV::Format->getDayName($day);
   }
   print "\">\n";

   print "<p>\n";
   foreach my $i ( @views ) {
      print "\t<a href=\"#".$i."\">".$views{$i}."</a><br/>\n";
   }
   print "\t<a href=\"#day\">"."Select day</a><br/>\n";
   print "\t<a href=\"#time\">"."Select timeslot</a><br/>\n";
   print "</p>\n";
   print "</card>\n";

# -- Channels cards
   foreach my $i ( @views ) {
      print "<card id=\"".$i."\" title=\"".$views{$i}."\">\n";
      print "<p>\n";
      foreach my $j ( 0 .. @{ $mapping{$i} }-1 ) {
         print "\t<a href=\"".$self."?go=list&amp;ch=".$mapping{$i}[$j].
            datetime()."\">".
            TV::Core->niceChannelName($mapping{$i}[$j])."</a><br/>\n";
      }
      print "\t<a href=\"".$self."?go=film&amp;view=".$i.
         datetime()."\">".
         "Films</a><br/>\n";
      print "</p>\n";
      print "</card>\n";
   }

# -- Day and time cards
   print "<card id=\"day\" title=\"Day Select\">\n";
   my @daylist = ("Yesterday","Today","Tomorrow");
   for (my $i=2; $i<7; $i++) {
      push(@daylist, TV::Format->getDayName($i));
   }
   print "<p>\n";
   for (my $i=0; $i<8; $i++) {
      print "\t<a href=\"".$self."?go=view&amp;day=".($i-1).
      "&amp;time=".$time."\">".$daylist[$i]."</a><br/>\n";
   }
   print "</p>\n";
   print "</card>\n";

   print "<card id=\"time\" title=\"Time select\">\n";
   print "<p>\n";
   foreach my $i (@slots) {
      print "\t<a href=\"".$self."?go=view&amp;day=".$day."&amp;time=".$i.
      "\">".$slots{$i}[0]."</a>".$slots{$i}[1]."<br/>\n";
   }
   print "</p>\n";
   print "</card>\n";

   print "</wml>\n";
   exit();
}

# -- Print the films in this view --------------------------------------
#
sub film {
   wml_header();
   wml_top($views{$view}." Films");

   my $c = TV::Search->new(days => [$day], channels => $mapping{$view});
   my $list = $c->findRegExp('Film', 0x08);

   foreach my $p (@$list) {
      print "\t\t<p><b><a href=\"".$self."?go=desc&amp;ch=".$p->{channel}.
         "&amp;pid=".$p->{pid}.datetime()."\">".(TV::Format->getTime($p))[0].
         "</a> ".$p->{title}." (".TV::Core->niceChannelName($p->{channel}).
         ")</b></p>\n";
   }

   wml_bottom();
   exit();
}

# -- Print the listing for a channel -----------------------------------
#
sub listing {
   wml_header();
   wml_top(TV::Core->niceChannelName($ch));

   my @array = (@{ $obj->{listings} });

   # We have to invent a 48 hour clock to make the logic simpler, 25:00 is
   # 1am tomorrow

   my $f = ($obj->getTime($array[0]))[0];
   my $o = 0;

   if ($tcols{$time}[0] ne "x") {
      foreach my $p (@array) {
         my $wtime = ($obj->getTime($p))[0];
         if ($wtime lt $f) {
            my @hm = split(/:/, $wtime);
            $wtime = ($hm[0] + 24).":".$hm[1];
         }
         if ($wtime ge $tcols{$time}[0]) { last; }
         $o++;
      }
      while ($o > 0) { shift @array; $o--; };
   }

   $o = 0;
   if ($tcols{$time}[1] ne "x") {
      foreach my $p (@array) {
         my $wtime = ($obj->getTime($p))[0];
         if ($wtime lt $f) {
            my @hm = split(/:/, $wtime);
            $wtime = ($hm[0] + 24).":".$hm[1];
         }
         if ($wtime gt $tcols{$time}[1]) { last; }
         $o++;
      }
      $o = $#array - $o + 1;
      while ($o > 0) { pop @array; $o--; };
   }

   foreach my $p (@array) {
      print "\t\t<p><b><a href=\"".$self."?go=desc&amp;ch=".$ch.
         "&amp;pid=".($obj->getPid($p)).datetime()."\">".($obj->getTime($p))[0].
         "</a> ".($obj->getSimpleTitle($p))."</b></p>\n";
   }

   wml_bottom();
   exit();
}

# -- Print the description for a programme -----------------------------
#
sub description {
   wml_header();
   wml_top($obj->getSimpleTitle(@{ $obj->{listings} }[$pid]));
   print "\t<p>".($obj->getTime(@{ $obj->{listings} }[$pid]))[0].
      " - ".($obj->getTime(@{ $obj->{listings} }[$pid]))[1].
      "</p>\n";
   print "\t<p>\n";
   print $obj->getDescription(@{ $obj->{listings} }[$pid])."\n";
   print "\t</p>\n";
   wml_bottom();
   exit();
}

# -- Print date and time url variables ---------------------------------
#
sub datetime {
   my $r = "";
   if ($day  != 0) {
      $r .= "&amp;day=".$day;
   }
   if ($time != 1) {
      $r .= "&amp;time=".$time;
   }
   return $r;
}

# -- Print a wml file header -------------------------------------------
#
sub wml_header {
   print "Content-type: text/vnd.wap.wml\n";
   if ($expire) {
      my $midnight = strftime "%a, %d %b %Y 23:59:59 GMT", localtime;
      print "Expires: ".$midnight."\n";
   }
print <<__EOFHTML__;

<?xml version="1.0"?>
<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.1//EN\"
\"http://www.wapforum.org/DTD/wml_1.1.xml\">
__EOFHTML__
}

# -- Print the top of a wml page ---------------------------------------
#
sub wml_top {
   print "<wml>\n";
   print "\t<card title=\"$_[0]\">\n";
}

# -- Print the bottom of a wml page ------------------------------------
#
sub wml_bottom {
   print "\t</card>\n";
   print "</wml>\n";
}
